
import sys

PR_OPENBLK  = 1
PR_CLOSEBLK = 2
PR_STRING   = 3
PR_BLANK    = 4
PR_NEWLINE  = 5
PR_EOF      = 6

# defaults are made large enough to not matter
prmargin = 200
prspace = 200
prblockindent = 4
prcontindent = 2
prlineno = 1
dirtyline = 0

class prutilstack:
    def __init__(self):
        self.st = []

    def reset(self):
        self.st = []

    def push(self, val):
        self.st.append(val)

    def pop(self):
        return self.st.pop()

    def isempty(self):
        return len(self.st) == 0

    def top(self):
        return self.st[len(self.st)-1]

class prvector:
    def __init__(self):
        self.vals = []

    def reset(self):
        self.vals = []

    def __getitem__(self, i):
        numvals= len(self.vals)
        while i >= numvals:
            self.vals.append(" ")
            numvals = len(numvals)
        return self.vals[i]
    def __setitem__(self, i, val):
        numvals = len(self.vals)
        while i >= numvals:
            self.vals.append(" ")
            numvals = len(self.vals)
        self.vals[i] = val

prs = prutilstack()
prs1 = prutilstack()

class PrBuffer:
    def __init__(self, channel=sys.stdout):
        self.buffer = ""
        self.bufferidx = 0
        self.channel = channel
    def nextchar(self):
        c = self.buffer[self.bufferidx]
        self.bufferidx = self.bufferidx + 1
        return c
    def putchar(self, c):
        self.bufferidx = self.bufferidx - 1
    def fullreset(self):
        global prlineno
        prlineno = 1
        prs.reset()
        prs1.reset()
        self.buffer = ""
        self.bufferidx = 0
    def reset(self):
        global prlineno
        prlineno = 1
        self.bufferidx = 0
    def fmtctrl(self, fmt):
        str = "%c"%(fmt)
        self.buffer = self.buffer + str
        self.bufferidx = self.bufferidx + len(str)
    def openblock(self, indent):
        self.fmtctrl(PR_OPENBLK)
        tens = int(indent/10)
        ones = indent%10
        self.buffer = self.buffer + chr(tens) + chr(ones)
        self.bufferidx = self.bufferidx + 2
    def closeblock(self):
        self.fmtctrl(PR_CLOSEBLK)
    def newline(self):
        self.fmtctrl(PR_NEWLINE)
    def substr(self, str):
        self.fmtctrl(PR_STRING)
        self.buffer = self.buffer + str + '\0'
        self.bufferidx = self.bufferidx + len(str) + 1
    def blank(self, cnt):
        self.fmtctrl(PR_BLANK)
        self.buffer = self.buffer + " "*cnt + '\0'
        self.bufferidx = self.bufferidx + cnt + 1
    def configure(self, margin, contindent, blockindent):
        global prspace
        global prmargin
        global prblockindent
        global prcontindent
        prmargin = margin
        prspace = margin
        prcontindent = contindent
        prblockindent = blockindent
    def setchannel(self, channel):
        self.channel = channel
    def outbuff(self):
        return self.channel
    def write(self, x):
        self.channel.write(x)

        
prbuff = PrBuffer()

def prindent(l, atmargin):
    global prmargin
    global prspace
    if dirtyline != 0:
        prbuff.write("\n")
        prbuff.write(" "*l)

def prformatted(s, l):
    global prspace
    global prmargin
    global prblockindent
    global prcontindent
    global prlineno
    global dirtyline

    if ord(s[0]) == PR_STRING:
        s = s[1:-1]     # strip control characters
        r = ""
        for c in s:
            r = r + c
#        if l > prspace:
#            prspace = prs.top() - 2
#            prindent(prmargin - prspace)
        prbuff.write(r)
        prspace = prspace - len(r)
        dirtyline = 1
    elif ord(s[0]) == PR_OPENBLK:
        tens = ord(s[1])
        ones = ord(s[2])
        indent = tens*10 + ones
        prspace = prmargin - indent
        prs.push(prspace)
        # do our own open
        prbuff.write("\n" + " "*(prmargin - prspace))
        # prindent(prmargin - prspace)
        prlineno = 1
        dirtyline = 0
    elif ord(s[0]) == PR_CLOSEBLK:
        prspace = prs.pop()
    elif ord(s[0]) == PR_NEWLINE:
        prspace = prs.top()
        prindent(prmargin - prspace, False)
        prlineno = 1
        dirtyline = 0
    elif ord(s[0]) == PR_BLANK:
        s = s[1:-1]     # strip control characters
        r = ""
        for c in s:
            r = r + c
        if len(r) > prspace:
            prbuff.write(" \\")
            prspace = prs.top() - prcontindent
            prindent(prmargin - prspace, False)
            prlineno = prlineno + 1
            dirtyline = 0
        else:
            prbuff.write(r)
            prspace = prspace - len(r)
            dirtyline = 1
    return

def prreceive():
    c = prbuff.nextchar()
    ret = []
    if ord(c) == PR_EOF:
        ret.append(chr(PR_EOF))
        return ret
    elif ord(c) == PR_OPENBLK:
        ret.append(chr(PR_OPENBLK))
        c = prbuff.nextchar()
        ret.append(c)
        c = prbuff.nextchar()
        ret.append(c)
        return ret
    elif ord(c) == PR_CLOSEBLK:
        ret.append(chr(PR_CLOSEBLK))
        return ret
    elif ord(c) == PR_NEWLINE:
        ret.append(chr(PR_NEWLINE))
        return ret
    elif ord(c) == PR_BLANK:
        ret.append(c)
        c = prbuff.nextchar()
        while ord(c) == ord(' '):
            ret.append(c)
            c = prbuff.nextchar()
        ret.append(c)      # trailing null
        return ret
    elif ord(c) == PR_STRING:
        ret.append(c)
        c = prbuff.nextchar()
        while ord(c) not in (PR_OPENBLK, PR_BLANK, PR_STRING, PR_NEWLINE, PR_CLOSEBLK):
            ret.append(c)
            c = prbuff.nextchar()
        prbuff.putchar(c)
        return ret

def prscan():
    global prspace
    global prmargin
    global prblockindent
    global prcontindent

    prsize = prvector()
    prstream = prvector()

    while True:
        x = prreceive()
        if ord(x[0]) == PR_EOF:
            break
        elif ord(x[0]) == PR_OPENBLK:
            if prs1.isempty():
                left = 1
                right = 1
                righttotal = 1
            else:
                right = right + 1
            prstream[right] = x
            prsize[right] = -righttotal
            prs1.push(right)
        elif ord(x[0]) == PR_CLOSEBLK:
            right = right + 1
            prstream[right] = x
            prsize[right] = 0
            idx = prs1.pop()
            prsize[idx] = righttotal + prsize[idx]
            if ord(prstream[idx][0]) == PR_BLANK:
                idx = prs1.pop()
                prsize[idx] = righttotal + prsize[idx]
            if prs1.isempty():
                while left < right:
                    prformatted(prstream[left], prsize[left])
                    left = left + 1
        elif ord(x[0]) == PR_BLANK or ord(x[0]) == PR_NEWLINE:
            right = right + 1
            idx = prs1.top()
            if ord(prstream[idx][0]) == PR_BLANK:
                prsize[prs1.pop()] = righttotal + prsize[idx]
            prstream[right] = x
            if ord(x[0]) == PR_BLANK:
                prsize[right] = -righttotal
                prs1.push(right)
                righttotal = righttotal + 1
            else:
                prsize[right] = 0
                # prs1.push(right)
        elif ord(x[0]) == PR_STRING:
            if prs1.isempty():
                prformatted(x, len(x) - 2)
            else:
                right = right + 1
                prstream[right] = x
                prsize[right] = len(x) - 2
                righttotal = righttotal + len(x) - 2
        else:
            pass

    return
