
import keyword
import os
import sys
import compiler
import traceback
import tempfile
import subprocess

import rwintfc

import sk2py

try:
    import wx
    import wx.stc as stc
except:
    print >> sys.stderr, "Can't find an proper installation of wxPython."
    print >> sys.stderr, "Get it at http://www.wxpython.org"
    import traceback
    traceback.print_exc()
    sys.exit()
    
    
ID_REWRITER=101
ID_OPEN=102
ID_XLATE=103
ID_SAVELEFT=104
ID_SAVERIGHT=105
ID_SAVELEFTAS=106
ID_SAVERIGHTAS=107
ID_PREFERENCES=108
ID_CUT=109
ID_COPY=110
ID_PASTE=111
ID_DELETE=112
ID_FIND=113
ID_FINDNEXT=114
ID_FINDPREV=115
ID_FINDREPLACE=116
ID_EXIT=200

# global values...
g_indentspaces = 4
g_prettyprint = False
g_pagewidth = 80
g_contindentspaces = 2
g_traceoutput = False

class Sk2PyLog(wx.PyLog):
    def __init__(self, textCtrl):
        wx.PyLog.__init__(self)
        self.tc = textCtrl

    def write(self, message):
        if self.tc:
            self.tc.AppendText(message + '\n')

#
# following for SkillSTC and PythonSTC are derived from stc.StyledTextCtrl_2
#

if wx.Platform == '__WXMSW__':
    faces = { 'times': 'Times New Roman',
              'mono' : 'Courier New',
              'helv' : 'Arial',
              'other': 'Comic Sans MS',
              'size' : 10,
              'size2': 8,
             }
else:
    faces = { 'times': 'Times',
              'mono' : 'Courier',
              'helv' : 'Helvetica',
              'other': 'new century schoolbook',
              'size' : 12,
              'size2': 10,
             }


class Preferences(wx.Dialog):
    "Preferences dialog."
    def __init__(self, Parent, id, title):
        global g_indentspaces
        global g_prettyprint
        global g_pagewidth
        global g_contindentspaces
        global g_traceoutput
        
        wx.Dialog.__init__(self, Parent, -1, "Preferences")

        self.statictext1 = wx.StaticText(self, -1, "Sk2Py Output Options")
        self.spaceline1 = wx.StaticLine(self, -1)

        self.indentspaces = wx.SpinCtrl(self, -1, "", size=(40, 20))
        self.indentspaces.SetRange(1,12)
        self.indentspaces.SetValue(g_indentspaces)
        self.statictext3 = wx.StaticText(self, -1, " block indent spaces")

        self.prettyprint = wx.CheckBox(self, -1, "Format Output")
        self.spaceline2 = wx.StaticLine(self, -1)
        self.Bind(wx.EVT_CHECKBOX, self.OnCheckBox, self.prettyprint)
        
        self.pagewidth = wx.SpinCtrl(self, -1, "", size=(50, 20))
        self.pagewidth.SetRange(1, 200)
        self.pagewidth.SetValue(g_pagewidth)
        self.statictext5 = wx.StaticText(self, -1, " total cols page width")

        self.contindentspaces = wx.SpinCtrl(self, -1, "", size=(40, 20))
        self.contindentspaces.SetRange(1, 12)
        self.contindentspaces.SetValue(g_contindentspaces)
        self.statictext7 = wx.StaticText(self, -1, " spaces indent for stmts")

        if g_prettyprint == True:
            self.prettyprint.SetValue(True)
            self.pagewidth.Enable(True)
            self.contindentspaces.Enable(True)
        else:
            self.prettyprint.SetValue(False)
            self.pagewidth.Enable(False)
            self.contindentspaces.Enable(False)

        self.statictext8 = wx.StaticText(self, -1, "Sk2Py Debug Options")
        self.spaceline3 = wx.StaticLine(self, -1)

        self.traceoutput = wx.CheckBox(self, -1, "Trace Output")
        if g_traceoutput == True:
            self.traceoutput.SetValue(True)
        else:
            self.traceoutput.SetValue(False)
        
        sizer1 = wx.BoxSizer(wx.HORIZONTAL)
        sizer1.Add(self.statictext1, 0, wx.ALIGN_LEFT)
        sizer1.Add(self.spaceline1, 1, wx.LEFT | wx.ALIGN_LEFT, 5)

        sizer2 = wx.BoxSizer(wx.HORIZONTAL)
        sizer2.Add((15, 10))
        sizer2.Add(self.indentspaces, 0, wx.LEFT | wx.ALIGN_LEFT, 5)
        sizer2.Add(self.statictext3, 0, wx.LEFT | wx.ALIGN_CENTER, 5)

        sizer3 = wx.BoxSizer(wx.HORIZONTAL)
        sizer3.Add((20, 10))
        sizer3.Add(self.prettyprint, 0, wx.ALIGN_CENTER)
        sizer3.Add(self.spaceline2, 1, wx.LEFT | wx.ALIGN_LEFT, 5)

        sizer4 = wx.BoxSizer(wx.HORIZONTAL)
        sizer4.Add((40, 10))
        sizer4.Add(self.pagewidth, 0, wx.LEFT | wx.ALIGN_LEFT, 5)
        sizer4.Add(self.statictext5, 0, wx.LEFT | wx.ALIGN_CENTER, 5)

        sizer5 = wx.BoxSizer(wx.HORIZONTAL)
        sizer5.Add((40, 10))
        sizer5.Add(self.contindentspaces, 0, wx.LEFT | wx.ALIGN_LEFT, 5)
        sizer5.Add(self.statictext7, 0, wx.LEFT | wx.ALIGN_CENTER, 5)

        sizer6 = wx.BoxSizer(wx.HORIZONTAL)
        sizer6.Add(self.statictext8, 0, wx.ALIGN_LEFT)
        sizer6.Add(self.spaceline3, 1, wx.LEFT | wx.ALIGN_CENTER, 5)

        sizer7 = wx.BoxSizer(wx.HORIZONTAL)
        sizer7.Add((20, 10))
        sizer7.Add(self.traceoutput, 0, wx.ALIGN_LEFT)
        
        self.OK = wx.Button(self, wx.ID_OK)
        self.OK.SetDefault()
        self.Cancel = wx.Button(self, wx.ID_CANCEL)
    
        sizer8 = wx.BoxSizer(wx.HORIZONTAL)
        sizer8.Add(self.OK, 0, wx.ALIGN_CENTER)
        sizer8.Add(self.Cancel, 0, wx.LEFT | wx.ALIGN_CENTER, 20)

        sizer9 = wx.BoxSizer(wx.VERTICAL)
        sizer9.Add(sizer1, 0, wx.EXPAND)
        sizer9.Add(sizer2, 0, wx.TOP, 5)
        sizer9.Add(sizer3, 0, wx.TOP | wx.EXPAND, 10)
        sizer9.Add(sizer4, 0, wx.TOP, 5)
        sizer9.Add(sizer5, 0, wx.TOP | wx.EXPAND, 10)
        sizer9.Add(sizer6, 0, wx.TOP | wx.EXPAND, 10)
        sizer9.Add(sizer7, 0, wx.TOP, 5)
        sizer9.Add(sizer8, 0, wx.TOP | wx.ALIGN_RIGHT, 10)

        sizer10 = wx.BoxSizer(wx.HORIZONTAL)
        sizer10.Add(sizer9, 1, wx.ALL, 10)

        self.SetSizerAndFit(sizer10)

    def OnOK(self, event):
        self.Close()

    def OnCancel(self, event):
        self.EndModal(wx.ID_CANCEL) 
    
    def OnCheckBox(self, event):
        cb = event.GetEventObject()
        if cb.GetValue() == True:
            self.prettyprint.SetValue(True)
            self.pagewidth.Enable(True)
            self.contindentspaces.Enable(True)
        else:
            self.prettyprint.SetValue(False)
            self.pagewidth.Enable(False)
            self.contindentspaces.Enable(False)

#----------------------------------------------------------------------

class SkillSTC(stc.StyledTextCtrl):
    def __init__(self, parent, parentframe, pos=wx.DefaultPosition, size=wx.DefaultSize, style=0):
        stc.StyledTextCtrl.__init__(self, parent, -1, pos, size, style) 

        self.parentframe = parentframe
        
        skillkeywords = (" t nil prog if then else let list when for "
          "foreach case return pcDefineCell integer float string "
          "procedure pcCellView car cdr cadr cddr")
        self.SetKeyWords(0, skillkeywords)
        self.SetLexerLanguage("lisp")

        self.SetMarginLeft(2)
        self.SetViewWhiteSpace(False)
        self.SetTabWidth(4)

        self.StyleDefault = 0
        self.StyleOne = 1

        self.SetMarginType(1, wx.stc.STC_MARGIN_NUMBER)
        self.SetMarginWidth(1, 30)
        
        # should do this only with pprinter
        # self.SetEdgeMode(stc.STC_EDGE_BACKGROUND)
        # self.SetEdgeColumn(78)

        self.StyleSetSpec(stc.STC_STYLE_DEFAULT,     "face:%(helv)s,size:%(size)d" % faces)
        self.StyleClearAll()  # Reset all to be like the default
        
        # Line numbers in margin
        self.StyleLineNumberSpec = "face:Courier New,size:10,fore:#000000,back:#F0F0F0"
        self.StyleSetSpec(wx.stc.STC_STYLE_LINENUMBER, self.StyleLineNumberSpec)
        # Global default styles for all languages
        self.StyleSetSpec(stc.STC_STYLE_DEFAULT,     "face:%(helv)s,size:%(size)d" % faces)
        self.StyleSetSpec(stc.STC_STYLE_LINENUMBER,  "back:#C0C0C0,face:%(helv)s,size:%(size2)d" % faces)
        self.StyleSetSpec(stc.STC_STYLE_CONTROLCHAR, "face:%(other)s" % faces)
        self.StyleSetSpec(stc.STC_STYLE_BRACELIGHT,  "fore:#FFFFFF,back:#0000FF,bold")
        self.StyleSetSpec(stc.STC_STYLE_BRACEBAD,    "fore:#000000,back:#FF0000,bold")

        # Python styles
        # Default 
        self.StyleSetSpec(stc.STC_P_DEFAULT, "fore:#000000,face:%(helv)s,size:%(size)d" % faces)
        # Comments
        self.StyleSetSpec(stc.STC_P_COMMENTLINE, "fore:#007F00,face:%(other)s,size:%(size)d" % faces)
        # Number
        self.StyleSetSpec(stc.STC_P_NUMBER, "fore:#007F7F,size:%(size)d" % faces)
        # String
        self.StyleSetSpec(stc.STC_P_STRING, "fore:#7F007F,face:%(helv)s,size:%(size)d" % faces)
        # Single quoted string
        self.StyleSetSpec(stc.STC_P_CHARACTER, "fore:#7F007F,face:%(helv)s,size:%(size)d" % faces)
        
        self.Bind(wx.EVT_MOUSE_EVENTS, self.OnMouse)
        self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
        
    def OnMouse(self, evt):
        if evt.ButtonDown():
            print "skill has focus"
            self.parentframe.EditorWithFocus(self, "skill")
        evt.Skip()
            
    def OnKeyDown(self, evt):
        self.parentframe.EditorModified(self, "skill")
        evt.Skip()

class PythonSTC(stc.StyledTextCtrl):
    def __init__(self, parent, parentframe, pos=wx.DefaultPosition, size=wx.DefaultSize, style=0):
        stc.StyledTextCtrl.__init__(self, parent, -1, pos, size, style) 

        self.parentframe = parentframe
        
        self.SetLexer(stc.STC_LEX_PYTHON)
        self.SetKeyWords(0, " ".join(keyword.kwlist))

        self.SetMarginLeft(2)
        self.SetViewWhiteSpace(False)
        self.SetTabWidth(4)

        self.StyleDefault = 0
        self.StyleOne = 1

        self.SetMarginWidth(1, 30)
        self.SetMarginType(1, wx.stc.STC_MARGIN_NUMBER)
         
        # self.SetIndentationGuides(True)
        
        # should do this only with pprinter
        # self.SetEdgeMode(stc.STC_EDGE_BACKGROUND)
        # self.SetEdgeColumn(78)

        self.StyleSetSpec(stc.STC_STYLE_DEFAULT,     "face:%(helv)s,size:%(size)d" % faces)
        self.StyleClearAll()  # Reset all to be like the default

        # Line numbers in margin
        self.StyleSetSpec(wx.stc.STC_STYLE_LINENUMBER,'fore:#000000,back:#99A9C2')
        # Global default styles for all languages
        self.StyleSetSpec(stc.STC_STYLE_DEFAULT,     "face:%(helv)s,size:%(size)d" % faces)
        self.StyleSetSpec(stc.STC_STYLE_LINENUMBER,  "back:#C0C0C0,face:%(helv)s,size:%(size2)d" % faces)
        self.StyleSetSpec(stc.STC_STYLE_CONTROLCHAR, "face:%(other)s" % faces)
        self.StyleSetSpec(stc.STC_STYLE_BRACELIGHT,  "fore:#FFFFFF,back:#0000FF,bold")
        self.StyleSetSpec(stc.STC_STYLE_BRACEBAD,    "fore:#000000,back:#FF0000,bold")

        # Python styles
        # Default 
        self.StyleSetSpec(stc.STC_P_DEFAULT, "fore:#000000,face:%(helv)s,size:%(size)d" % faces)
        # Comments
        self.StyleSetSpec(stc.STC_P_COMMENTLINE, "fore:#007F00,face:%(other)s,size:%(size)d" % faces)
        # Number
        self.StyleSetSpec(stc.STC_P_NUMBER, "fore:#007F7F,size:%(size)d" % faces)
        # String
        self.StyleSetSpec(stc.STC_P_STRING, "fore:#7F007F,face:%(helv)s,size:%(size)d" % faces)
        # Single quoted string
        self.StyleSetSpec(stc.STC_P_CHARACTER, "fore:#7F007F,face:%(helv)s,size:%(size)d" % faces)
        # Keyword
        self.StyleSetSpec(stc.STC_P_WORD, "fore:#00007F,bold,size:%(size)d" % faces)
        # Triple quotes
        self.StyleSetSpec(stc.STC_P_TRIPLE, "fore:#7F0000,size:%(size)d" % faces)
        # Triple double quotes
        self.StyleSetSpec(stc.STC_P_TRIPLEDOUBLE, "fore:#7F0000,size:%(size)d" % faces)
        # Class name definition
        self.StyleSetSpec(stc.STC_P_CLASSNAME, "fore:#0000FF,bold,underline,size:%(size)d" % faces)
        # Function or method name definition
        self.StyleSetSpec(stc.STC_P_DEFNAME, "fore:#007F7F,bold,size:%(size)d" % faces)
        # Operators
        self.StyleSetSpec(stc.STC_P_OPERATOR, "bold,size:%(size)d" % faces)
        # Identifiers
        self.StyleSetSpec(stc.STC_P_IDENTIFIER, "fore:#000000,face:%(helv)s,size:%(size)d" % faces)
        # Comment-blocks
        self.StyleSetSpec(stc.STC_P_COMMENTBLOCK, "fore:#7F7F7F,size:%(size)d" % faces)
        # End of line where string is not closed
        self.StyleSetSpec(stc.STC_P_STRINGEOL, "fore:#000000,face:%(mono)s,back:#E0C0E0,eol,size:%(size)d" % faces)
        
        self.Bind(wx.EVT_MOUSE_EVENTS, self.OnMouse)
        self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)

    def OnMouse(self, evt):
        if evt.ButtonDown():
            self.parentframe.EditorWithFocus(self, "python")
        evt.Skip()

    def OnKeyDown(self, evt):
        self.parentframe.EditorModified(self, "python")
        evt.Skip()
            
class RWIntfcEditDlg(wx.Dialog):
    def __init__(self, parent, rwfile):
        wx.Dialog.__init__(self, id=-1,
              name='RWIntfcEditDlg', parent=parent,
              size=wx.Size(600, 500),
              style=wx.WANTS_CHARS | wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER,
              title="Rewriter Interface Object Editor")
        
        self.text = PythonSTC(self, style=wx.TE_MULTILINE)

        self.SetClientSize(wx.Size(600, 500))
        self.SetAutoLayout(True)
        self.SetSizeHints(650, 550, -1, -1)
        self.Center(wx.BOTH)
        
        gap = wx.LEFT | wx.TOP | wx.RIGHT
        
        self.okButton = wx.Button(self, wx.ID_OK, "OK")
        self.cancelButton = wx.Button(self, wx.ID_CANCEL, "Cancel")

        btnSizer = wx.BoxSizer(wx.HORIZONTAL)
        btnSizer.Add(self.okButton,     0, gap, 5)
        btnSizer.Add(self.cancelButton, 0, gap, 5)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.text, 1, gap | wx.EXPAND,       5)
        sizer.Add((10, 10))
        sizer.Add(wx.StaticLine(self))
        sizer.Add(btnSizer,      0, gap | wx.ALIGN_CENTRE, 5)
        sizer.Add((10, 10))

        self.SetAutoLayout(True)
        self.SetSizer(sizer)
        sizer.Fit(self)

        self.text.SetFocus()
 
    def loadFile(self, rwfile):
        f = open(rwfile, 'r')
        t = f.read()
        newt = os.linesep.join(l.rstrip() for l in t.split(os.linesep))
        newt.expandtabs(4)
        self.text.AddText(newt)
        f.close()
        
    def saveFile(self, rwfile):
        f = open(rwfile, "w")
        t = self.text.GetText()
        newt = os.linesep.join(l.rstrip() for l in t.split(os.linesep))
        newt.expandtabs(4)
        f.write(newt)
        f.close()
            
               
########################################################################################
#
# redirects generated python code to the gui w/ or w/o linenumbers
# (not using linenumber interface now)

class GuiRedirect(sk2py.Redirect):
    def __init__(self, fr):
        sk2py.Redirect.__init__(self)
        self.fr = fr
        self.pylineno = 0
        self.lastsklineno = -1
        numlines = self.fr.skilleditor.GetLineCount()
        for i in range(0, 2*numlines):
            self.fr.linemap.append(-1)
        
    def write(self, s):
        self.fr.pycelleditor.AppendText(s)
        
    def buff(self, s, sklineno=-1):
        lines = 1 + s.count("\n")
        self.pylineno = self.pylineno + lines
        if sklineno > 0:
            self.fr.linemap[sklineno] = self.pylineno
            self.lastsklineno = sklineno
        elif self.lastsklineno > 0:
            sklineno = self.lastsklineno
        # print "skill lineno = %d, python lineno = %d"%(sklineno, self.pylineno)
        self.fr.pycelleditor.AppendText(s + "\n")
  
########################################################################################
# can't do this with StaticText - so must do dynamic labels ourselves

class DynamicLabel(wx.Window):
    def __init__(self, parent, title, pos=(5,5), style=wx.NO_BORDER, size=wx.DefaultSize):
        wx.Window.__init__(self, parent)
        self.text = "(none)"
        self.title = title
        self.style = style
        self.annotate = " "        
        self.Bind(wx.EVT_PAINT, self.OnPaint)

    def OnPaint(self, evt):
        sz = self.GetSize()
        dc = wx.PaintDC(self)
        dc.SetBackground(wx.Brush(self.GetBackgroundColour()))
        dc.SetTextForeground("black")
        dc.Clear()
        prefix = self.title + " : "
        label = prefix + self.text + " " + self.annotate
        w,h = dc.GetTextExtent(label)
        if self.style & wx.ALIGN_RIGHT:
            x = sz.width - w
        else:
            x = (sz.width - w)/2
            
        # "face:%(helv)s,size:%(size)d" % faces)
        dc.DrawText(label, 5, (sz.height - h)/2)
        
    def TagModified(self):
        self.annotate = "*"
        self.Refresh()
    
    def TagUnmodified(self):
        self.annotate = " "
        self.Refresh()
    
    def SetLabel(self, text):
        self.text = text
        self.Refresh()
        
    def GetLabel(self):
        return self.text

########################################################################################

class LogWindow(wx.TextCtrl):
    def __init__ (self, parent):
        wx.TextCtrl.__init__(self, parent, -1, \
            style=wx.TE_MULTILINE | wx.TE_READONLY)
        self.redirect()
        
    def redirect(self, option=''):
        if option == "reset":
            sys.stdout = sys.__stdout__
            sys.stderr = sys.__stderr__
        else:
            sys.stdout = self
            sys.stderr = self
    
    def ClearAll(self):
        self.ClearAll()

########################################################################################

python_file=""

class MainWindow(wx.Frame):
    def __init__(self,parent,id,title):
        self.skillfilenamepath = ''     # full path + filename
        self.pycellfilenamepath = ''    # full path + filename
        self.skdirname = ''     # directory name only
        self.pydirname = ''     # directory name only
        self.skfilename = ''    # file name only
        self.pyfilename = ''    # file name only
        self.pycellfilesaved = False
        
        self.compprocess = ''
        self.focuseditor = None
        
        wx.Frame.__init__(self, parent, wx.ID_ANY, title, size=(800,500))
        
        self.topsplitter = wx.SplitterWindow(self, -1, \
            style=wx.CLIP_CHILDREN|wx.SP_LIVE_UPDATE|wx.SP_3D)
        self.editsplitter = wx.SplitterWindow(self.topsplitter, -1, \
            style=wx.CLIP_CHILDREN|wx.SP_LIVE_UPDATE|wx.SP_3D)
        self.log = LogWindow(self.topsplitter)
        
        self.skilleditor = SkillSTC(self.editsplitter, self, style=wx.TE_MULTILINE)
        self.pycelleditor = PythonSTC(self.editsplitter, self, style=wx.TE_MULTILINE)
        self.CreateStatusBar()

        self.editsplitter.SplitVertically(self.skilleditor, self.pycelleditor, 0)
        self.editsplitter.SetSashGravity(0.5)
        self.topsplitter.SplitHorizontally(self.editsplitter, self.log, -120)
        
        self.topsplitter.SetMinimumPaneSize(20)
        self.editsplitter.SetMinimumPaneSize(200)
         
        self.finddlg = None
        self.finddata = wx.FindReplaceData()
        self.finddata.SetFlags(wx.FR_DOWN)
        
        # make editsplitter expand on resize
        def SplitterOnSize(evt):
            splitwin = evt.GetEventObject()
            sz = splitwin.GetSize()
            # 120 is arbitrary - should save the amount on resize of h-sash
            splitwin.SetSashPosition(sz.height - 120, False)
            evt.Skip()
            
        self.topsplitter.Bind(wx.EVT_SIZE, SplitterOnSize)
                               
        # self.controlBox = wx.BoxSizer(wx.HORIZONTAL)
        self.controlBox = wx.FlexGridSizer(rows=1, cols=3, hgap=2, vgap=2)
        self.controlBox.AddGrowableCol(0, 1)
        # self.controlBox.AddGrowableCol(1, 0)
        
        self.btnXlate = wx.Button(self, -1, "Gen PyCell")
        self.btnXlate.Enable(False)
        self.btnXlate.Bind(wx.EVT_BUTTON, self.OnXlate)
        self.btnCompile = wx.Button(self, -1, "Compile PyCell")
        self.btnCompile.Enable(False)
        self.btnCompile.Bind(wx.EVT_BUTTON, self.OnCompile)

        self.skillfilenamedisplay = DynamicLabel(self, "Skill", style=wx.ALIGN_RIGHT)
        self.pycellfilenamedisplay = DynamicLabel(self, "Python", style=wx.ALIGN_RIGHT)
        
        # .Add([window,sizer,size],proportion=0,flag=0,border=0,userData=none)
        self.filenameBox = wx.BoxSizer(wx.VERTICAL)
        self.filenameBox.Add(self.skillfilenamedisplay, 0, wx.EXPAND | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 2)
        self.filenameBox.Add(self.pycellfilenamedisplay, 0, wx.EXPAND | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 2)
        self.controlBox.Add(self.filenameBox, 0, wx.EXPAND | wx.ALIGN_LEFT | wx.ALIGN_CENTER_VERTICAL, 2)
        # self.controlBox.Add(wx.Size(1, 1), 0, wx.ALIGN_RIGHT | wx.FIXED_MINSIZE, 2)
        self.controlBox.Add(self.btnXlate, 0, wx.EXPAND, wx.ALIGN_RIGHT | wx.FIXED_MINSIZE | wx.ALIGN_CENTER_VERTICAL, 2)
        self.controlBox.Add(self.btnCompile, 0, wx.EXPAND, wx.ALIGN_RIGHT | wx.FIXED_MINSIZE | wx.ALIGN_CENTER_VERTICAL, 2)
        self.controlBox.Fit(self)
        
        self.box = wx.BoxSizer(wx.VERTICAL)
        self.box.Add(self.controlBox, 0, wx.EXPAND)
        self.box.Add(wx.StaticLine(self), 0, wx.EXPAND)
        self.box.Add(self.topsplitter, 1, wx.EXPAND)
        
        self.SetMinSize(wx.Size(800,500))
        self.SetSizer(self.box)
        self.box.Fit(self)
        
        self.linemap=[]

        self.skismodified = False
        self.pyismodified = False
        
        # Setting up the menu.       
        filemenu= wx.Menu()
        filemenu.Append(ID_OPEN, "&Open"," Open A Skill PCell")
        filemenu.AppendSeparator()
        filemenu.Append(ID_SAVELEFT, "Save &Skill"," Save Skill")
        filemenu.Append(ID_SAVELEFTAS, "Save Skill As"," Save Skill As")
        filemenu.AppendSeparator()
        filemenu.Append(ID_SAVERIGHT, "Save &Python"," Save Python")
        filemenu.Append(ID_SAVERIGHTAS, "Save Python As"," Save Python As")
        filemenu.AppendSeparator()
        filemenu.Append(ID_REWRITER, "&Rewriter"," Update rewriter objects")
        filemenu.AppendSeparator()
        filemenu.Append(ID_EXIT,"E&xit"," Terminate the program")
        
        editmenu = wx.Menu()
        editmenu.Append(ID_CUT, "&Cut", " cut")
        editmenu.Append(ID_COPY, "&Copy", " copy")
        editmenu.Append(ID_PASTE, "&Paste", " paste")
        editmenu.Append(ID_DELETE, "&Delete", " delete")
        editmenu.AppendSeparator()
        editmenu.Append(ID_FIND, "&Find\tCTRL-F", " find")
        editmenu.Append(ID_FINDNEXT, "Find &Next\tF3", " find next")
        editmenu.AppendSeparator()
        editmenu.Append(ID_PREFERENCES, "&Preferences"," Program preferences")
                
        # Creating the menubar.
        menuBar = wx.MenuBar()
        menuBar.Append(filemenu,"&File")
        menuBar.Append(editmenu,"&Edit")
        
        self.SetMenuBar(menuBar)
        
        wx.EVT_MENU(self, ID_REWRITER, self.OnRewriter)
        wx.EVT_MENU(self, ID_OPEN, self.OnOpen)
        wx.EVT_MENU(self, ID_SAVELEFT, self.OnSaveLeft)
        wx.EVT_MENU(self, ID_SAVELEFTAS, self.OnSaveLeftAs)
        wx.EVT_MENU(self, ID_SAVERIGHT, self.OnSaveRight)
        wx.EVT_MENU(self, ID_SAVERIGHTAS, self.OnSaveRightAs)
        wx.EVT_MENU(self, ID_PREFERENCES, self.OnPrefDialog)
        wx.EVT_MENU(self, ID_EXIT, self.OnExit)
        wx.EVT_MENU(self, ID_CUT, self.OnCut)
        wx.EVT_MENU(self, ID_COPY, self.OnCopy)
        wx.EVT_MENU(self, ID_PASTE, self.OnPaste)
        wx.EVT_MENU(self, ID_DELETE, self.OnDelete)
        wx.EVT_MENU(self, ID_FIND, self.OnHelpFind)
        wx.EVT_MENU(self, ID_FINDNEXT, self.OnFind)
        
        self.Bind(wx.EVT_FIND, self.OnFind)
        self.Bind(wx.EVT_FIND_NEXT, self.OnFind)
#        self.Bind(wx.EVT_FIND_PREV, self.OnFind)
        self.Bind(wx.EVT_FIND_CLOSE, self.OnFindClose)
        # self.Bind(wx.EVT_FIND_REPLACE, self.OnFindReplace)
        
        self.Show(1)
        
    #def OnLeftDown(self, event):
    # to allow mapping between the left and right views.  problem is,
    # if either of the views are edited, this becomes difficult (as if
    # it already wasn't difficult).
    #
    #    x = event.m_x
    #    y = event.m_y
    #    line = (y + self.skilleditor.GetScrollPos(wx.VERTICAL))/self.skilleditor.GetCharHeight()
    #    self.SetTitle('right mouse line # = ' + str(line))
    #    if self.linemap[line] != -1:
    #        self.pycelleditor.GotoLine(self.linemap[line])
    #        startpos = self.pycelleditor.PositionFromLine(self.linemap[line])
    #        endpos = self.pycelleditor.PositionFromLine(self.linemap[line] + 1)
    #        self.pycelleditor.SetSelection(startpos, endpos - 1)
         
        
    def EditorModified(self, editor, name):
        if name == "skill":
            self.skillfilenamedisplay.TagModified()
        else:
            self.pycellfilenamedisplay.TagModified()
        # print "sk2py: editor modified: " + name
    
    def EditorWithFocus(self, editor, name):
        self.focuseditor = editor
        # update is-mofified indicators in case we missed something (such as cut-n-paste)
        if self.skilleditor.GetModify():
            self.skillfilenamedisplay.TagModified()
        if self.pycelleditor.GetModify():
            self.pycellfilenamedisplay.TagModified()
            
        # print "sk2py: editor with focus: " + name
        
    def OnAbout(self, e):
        res = wx.MessageBox("Skill->Python\nPCell Migration Tool", "sk2py", wx.OK)

    def OnRewriter(self,e):      
        rwfile = rwintfc.__file__
        if rwfile[-1] == 'c':
            # strip off the c of .pyc for compiled modules (seem to get sometimes).
            rwfile = rwfile[:-1]
        self.log.WriteText("Loading interface objects from: " + rwfile + "\n")
        dlg = RWIntfcEditDlg(self, rwfile)
        dlg.loadFile(rwfile)
        dlg.CenterOnScreen()
        val = dlg.ShowModal()
        if val == wx.ID_OK:
            # save the changes.
            dlg.saveFile(rwfile)
            sk2py.reloaddefs();
            if self.skillfilenamepath != "":
                self.btnXlate.Enable(True)
            self.log.WriteText("Rewriter interface objects updated.\n")
        else:
            self.log.WriteText("Update for rewriter objects cancelled.\n")
        dlg.Destroy()
        
    def OnPrefDialog(self, event):
        global g_indentspaces
        global g_prettyprint
        global g_pagewidth
        global g_contindentspaces
        global g_traceoutput

        dia = Preferences(self, -1, "Preferences")
        dia.CenterOnScreen()
        val = dia.ShowModal()    
        if val == wx.ID_OK:
            g_indentspaces = dia.indentspaces.GetValue()
            g_prettyprint = dia.prettyprint.GetValue()
            g_pagewidth = dia.pagewidth.GetValue()
            g_contindentspaces = dia.contindentspaces.GetValue()
            g_traceoutput = dia.traceoutput.GetValue()
            self.btnXlate.Enable(True)

        dia.Destroy()
        
    def OnOpen(self,e):
        # probably should do niceties like see if a file is open, needs to
        # be saved - things like that.
        dlg = wx.FileDialog(self, "Choose a file", self.skdirname, "", "*.il;*.*", wx.OPEN)
        if dlg.ShowModal() == wx.ID_OK:
            self.skfilename = dlg.GetFilename()
            self.skdirname = dlg.GetDirectory()
            self.skillfilenamepath = os.path.join(self.skdirname, self.skfilename)
            self.pycellfilesaved = False
            
            # only update potential python directory and filenames
            if self.pydirname == '':
                self.pydirname = self.skdirname
            root, ext = os.path.splitext(self.skfilename)
            self.pyfilename = root + ".py"
            pycellfilenamepath = ''
                
            self.skillfilenamedisplay.SetLabel(self.skillfilenamepath)

            f = open(self.skillfilenamepath,'r')
            self.skilleditor.ClearAll()
            self.pycelleditor.ClearAll()
            self.log.Clear()
            self.skilleditor.AddText(f.read())
            self.btnXlate.Enable(True)
            self.skismodified = True
            f.close()
        dlg.Destroy()
                    
    def OnXlate(self,e):
        global g_indentspaces
        global g_prettyprint
        global g_pagewidth
        global g_contindentspaces
        global g_traceoutput
        if self.skillfilenamepath != "":
            wx.BeginBusyCursor()
            redir = GuiRedirect(self)
            tempfilename = tempfile.mktemp()
            f = open(tempfilename, "w")
            f.write(self.skilleditor.GetText())
            f.close()
            self.pycelleditor.ClearAll()
            self.log.Clear()
            
            try:
                sk2py.guiintfc(tempfilename, redir, g_indentspaces, g_prettyprint, \
                    g_pagewidth, g_contindentspaces, g_traceoutput)
            except:
                wx.EndBusyCursor()
            else:
                os.remove(tempfilename)
                wx.EndBusyCursor()
                self.btnXlate.Enable(False)
                self.pyismodified = True
                
        self.btnCompile.Enable(True)

    def OnCompile(self, evt):
        self.btnCompile.Enable(False)
        self.pyismodified = True

        print "do compile"

        if self.skillfilenamepath != "":
            wx.BeginBusyCursor()
            redir = GuiRedirect(self)

            if wx.Platform == '__WXMSW__':
                print "syntax check (windows)"
                try:
                    tempfilename = tempfile.mktemp()
                    f = open(tempfilename, "w")
                    f.write(self.pycelleditor.GetText())
                    f.close()
                    
                    filestr = open(tempfilename).read()
                    python_file = tempfilename
                    compile(filestr, python_file, "exec")
                except SyntaxError:
                    ev = sys.exc_info()[1]
                    for k, v in vars(ev).items():
                        print k, "=", repr(v)
                else:
                    print "%s: compiles OK."%python_file
                    os.remove(tempfilename)
            else:   
                print "compile with cngenlib (linux)"
                
                if self.pyismodified:
                    dlg = wx.MessageDialog(self, 'You must save PyCell to the S~/k2PyCells directory first.',
                        'Compilation Error!', wx.OK | wx.ICON_INFORMATION)
                    dlg.ShowModal()
                    dlg.Destroy()
                    wx.EndBusyCursor()
                    return
                
                p,f = os.path.split(self.pycellfilenamepath)
                p,d = os.path.split(p)
                if p != "Sk2PyCells":
                    dlg = wx.MessageDialog(self, 'You must save PyCell to the S~/k2PyCells directory first.',
                        'Compilation Error!', wx.OK | wx.ICON_INFORMATION)
                    dlg.ShowModal()
                    dlg.Destroy()
                    wx.EndBusyCursor()
                    return                    
                   
                # the following should be parameters:
                #   pkg name of python files
                #   python library name
                #   python library path
                #   technology file path
                #   option to create library (first time)
                #   option to view cell
                #   need to update the __init__.py and lib.defs files
                #
                try:
                    home = os.curdir
                    if 'HOME' in os.environ:
                        home = os.environ['HOME']
                    elif os.name == 'posix':
                        home = os.path.expanduser("~/")     
                    #
                    # need to save the module under the Sk2PyCells directory
                    # (as well as update __init__.py, lib.defs, etc.) - but not now.
                    #            
                    cnhome = os.environ['CNI_ROOT']
                    cngenlib = os.path.join(cnhome, "bin", "cngenlib")                    
                    cmd = cngenlib + " --techfile " + os.path.join(home, "Sk2PyTechLib", "Santana.tech")
                    cmd += " pkg:Sk2PyCells Sk2PyCellLib " + os.path.join(home, "Sk2PyCellLib")
                    
                    p = subprocess.Popen(cmd, shell=True, bufsize=8192, stdin=None, \
                        stdout=subprocess.PIPE, stderr=subprocess.PIPE, close_fds=True)
                    self.log.write(p.stdout.read())
                    self.log.write(p.stderr.read())
                except:
                    print "error during compilation"
                    self.btnXlate.Enable(True)
                else:
                    self.btnXlate.Enable(False)
                    

            wx.EndBusyCursor()
            self.btnXlate.Enable(False)
    
    def OnExit(self, evt):
        self.Close(True)

    def OnSkillModified(self, evt):
        self.btnXlate.Enable(True)
        self.skismodified = True
        
    def OnPycellModified(self, evt):
        self.btnCompile.Enable(True)
        self.pyismodified = True
        
    def OnSaveLeft(self, evt):
        textfile = open(os.path.join(self.skdirname, self.skfilename), 'w')
        textfile.write(self.skilleditor.GetText())
        textfile.close()
        self.skismodified = False
        self.skilleditor.SetSavePoint()
        self.skillfilenamedisplay.TagUnmodified()

    def OnSaveRight(self, evt):
        if self.pycellfilenamepath == '':
            if self.askUserForFilename(True, defaultFile=self.pyfilename, \
                style = wx.SAVE, **self.defaultPythonDialogOptions()):
                self.pycellfilenamepath = os.path.join(self.pydirname, self.pyfilename)
                textfile = open(self.pycellfilenamepath, 'w')
                textfile.write(self.pycelleditor.GetText())
                textfile.close()    
                self.pycellfilenamedisplay.TagUnmodified()
                self.pycellfilenamedisplay.SetLabel(self.pycellfilenamepath)
                self.pyismodified = False
                self.pycelleditor.SetSavePoint()
        else:
            textfile = open(os.path.join(self.pydirname, self.pyfilename), 'w')
            textfile.write(self.pycelleditor.GetText())
            textfile.close()
            self.pyismodified = False       
            self.pycellfilenamedisplay.TagUnmodified()  
            self.pycellfilenamedisplay.SetLabel(self.pycellfilenamepath)
            self.pycelleditor.SetSavePoint()

    def OnSaveLeftAs(self, evt):
        if self.askUserForFilename(False, defaultFile=self.skfilename, \
            style = wx.SAVE, **self.defaultSkillDialogOptions()):
            textfile = open(os.path.join(self.skdirname, self.skfilename), 'w')
            textfile.write(self.skilleditor.GetText())
            textfile.close()
            self.skismodified = False
            self.skilleditor.SetSavePoint()
            self.skillfilenamedisplay.TagUnmodified()

    def OnSaveRightAs(self, evt):
        if self.askUserForFilename(True, defaultFile="", \
            style = wx.SAVE, **self.defaultPythonDialogOptions()):
            self.pycellfilenamepath = os.path.join(self.pydirname, self.pyfilename)
            textfile = open(self.pycellfilenamepath, 'w')
            textfile.write(self.pycelleditor.GetText())
            textfile.close()
            self.pycellfilenamedisplay.SetLabel(self.pycellfilenamepath)
            self.pyismodified = False
            self.pycelleditor.SetSavePoint()
            self.pycellfilenamedisplay.TagUnmodified()

    def defaultSkillDialogOptions(self):
        return dict(message='Choose a file', defaultDir=self.skdirname,
            wildcard='*.il')

    def defaultPythonDialogOptions(self):
        return dict(message='Choose a file', defaultDir=self.pydirname,
            wildcard='*.py')
                        
    def askUserForFilename(self, ispython, **dialogOptions):
        dialog = wx.FileDialog(self, **dialogOptions)
        if dialog.ShowModal() == wx.ID_OK:
            userProvidedFilename = True
            if ispython == True:
                self.pyfilename = dialog.GetFilename()
                self.pydirname = dialog.GetDirectory()
            else:
                self.skfilename = dialog.GetFilename()
                self.skdirname = dialog.GetDirectory()
        else:
            userProvidedFilename = False
        dialog.Destroy()
        return userProvidedFilename
    
    def OnFind(self, event):
        if self.focuseditor == None:
            self.focuseditor = self.skilleditor
                    
        textstring = self.focuseditor.GetText().lower()
        findstring = self.finddata.GetFindString().lower()
        backward = not (self.finddata.GetFlags() & wx.FR_DOWN)
        loc = -1
        if backward:
            start = self.focuseditor.GetSelection()[0]
            loc = textstring.rfind(findstring, 0, start)
        else:
            start = self.focuseditor.GetSelection()[1]
            loc = textstring.find(findstring, start)
        if loc == -1 and start != 0:
            # string not found, start at beginning
            if backward:
                start = end
                loc = textstring.rfind(findstring, 0, start)
            else:
                start = 0
                loc = textstring.find(findstring, start)
        if loc == -1:
            msg = "String not found: \"" + findstring + "\""
            dlg = wx.MessageDialog(self, msg, 'Find', wx.OK | wx.ICON_INFORMATION)
            dlg.ShowModal()
            dlg.Destroy()
        if self.finddlg:
            if loc == -1:
                self.finddlg.SetFocus()
                return
            else:
                self.finddlg.Destroy()
                self.finddlg = None
        self.focuseditor.ShowLines(self.focuseditor.LineFromPosition(loc), self.focuseditor.LineFromPosition(loc))
        self.focuseditor.SetSelection(loc, loc + len(findstring))

    def OnFindNext(self, event):
        if self.finddata.GetFindString():
            self.OnFind(event)
        else:
            self.OnHelpFind(event)

    def OnFindPrev(self, event):
        if self.finddata.GetFindString():
            self.OnFind(event)
        else:
            self.OnHelpFind(event)

    def OnFindClose(self, event):
        event.GetDialog().Destroy()
        self.finddlg = None

    def OnHelpFind(self, event):
        if self.finddlg != None:
            return
        
        if self.focuseditor == None:
            self.focuseditor = self.skilleditor
                
        selection = self.focuseditor.GetSelectedText()        
        self.finddlg = wx.FindReplaceDialog(self, self.finddata, "Find",
                        wx.FR_NOMATCHCASE | wx.FR_NOWHOLEWORD)
        self.finddlg.data = self.finddata
        self.finddlg.Show(True)
        
    def OnPaste(self,event):
        if self.focuseditor == None:
            return

        selection = self.focuseditor.GetSelectedText()
        self.focuseditor.Paste()

    def OnCopy(self,event):
        if self.focuseditor == None:
            return
                
        selection = editor.GetSelectedText()
        self.focuseditor.Copy()

        if selection:
            if not wx.TheClipboard.Open():
                logger.error("error opening system clipboard")
                return
            data = wx.TextDataObject(selection)
            wx.TheClipboard.AddData(data)
            wx.TheClipboard.Close()        

    def OnDelete(self,event):
        if self.focuseditor == None:
            return
                
        self.focuseditor.Cut()
        
    def OnCut(self,event):
        if self.focuseditor == None:
            return
                
        selection = self.focuseditor.GetSelectedText()
        self.focuseditor.Cut()

        if selection:
            if not wx.TheClipboard.Open():
                logger.error("error opening system clipboard")
                return
            data = wx.TextDataObject(selection)
            wx.TheClipboard.AddData(data)
            wx.TheClipboard.Close()
            
        
app = wx.PySimpleApp()
frame = MainWindow(None, -1, "Sk2Py Migration Tool")
app.MainLoop()
