
#
# methods that can be incorporated when needed into a 
# translated modules definition
#


moduleDefRodCreatePath = """

    def rodCreatePath(self, layer=None, cvId=None, width=None, pts=[], justification="center", subRect=[[]]):
        global grpcnt
        grpname = "grp%d"%(grpcnt)
        grpcnt = grpcnt + 1
        g = Grouping(grpname)

        print "*************************************"
        print "rodCreatePath: %s"%(grpname)

        # use the path object - easier and cleaner
        pr = Path(Layer(layer), width=width, points=pts)
        g.add(pr)

        if subRect == None:
            return g

        #
        # subrects are specified, need to populate each segment of 
        # the path, according to the spacing requirements.
        #
        prevXOffset = 0
        prevYOffset = 0
        p0 = pts[0]
        for pidx in range(1, len(pts)):
            p1 = pts[pidx]
            print "p0 = ",p0
            print "p1 = ",p1

            # two assumptions here - first definately needs addressing
            # 1) assume directions are pos-x and pos-y directions.
            # 2) assume segments are orthogonal
            #
            seqPrev = False
            seqNext = False
            if pidx < len(pts) - 1:
                seqNext = True
            if pidx > 1:
                seqPrev = True

            # determine direction of segment
            x1 = p0.getX()
            y1 = p0.getY()
            x2 = p1.getX()
            y2 = p1.getY()
            if x1 == x2 and y1 == y2:
                print "error: coincident points on path"
                return g
            dir = None
            if x1 == x2:
                dir = 'ydir'
            if y1 == y2:
                dir = 'xdir'
            if dir == None:
                print "error determining direction of path"
                return g
            print "direction: %s"%(dir)

            # go through subrects to determine extent 
            maxSpacing = 0.0
            maxSrLength = 0.0

            # initialize subrect variables outside of the loop
            # for individual subrects.  it seems as if some
            # parameters are carried over from subrect-subrect
            # if not specified.  this is apparent from looking
            # at multi-layer vias where the offsets are initially
            # specified for lower layer, then 'defaulted' for
            # upper layers.  should verify with experiments.
            #
            srlayer = None
            srLength = 0
            srWidth = 0
            srSpace = 0
            srBeginOffset = 0
            srEndOffset = 0
            for sr in subRect:
                for par in sr:
                    # the parameters are passed in a tuple
                    param,val = par
                    print par
                    if param == "layer":
                        srLayer = val
                    elif param == "length":
                        srLength = val
                    elif param == "width":
                        srWidth = val
                    elif param == "space":
                        srSpace = val
                    elif param == "beginOffset":
                        srBeginOffset = val
                    elif param == "endOffset":
                        srEndOffset = val
                    
                # initialize dependent parameters
                if srLength == 0:
                    srLength = srWidth
                if srWidth == 0:
                    srWidth = srLength
                if srEndOffset == 0:
                    srEndOffset = srBeginOffset
                if srBeginOffset == 0:
                    srBeginOffset = srEndOffset
                    
                print "layer = ", srLayer
                print "length = ", srLength
                print "width = ", srWidth
                print "space = ", srSpace
                print "beginOffset = ", srBeginOffset
                print "endOffset = ", srEndOffset

                maxSrLength = float(max(maxSrLength, srLength))
                maxSpacing = float(max(maxSpacing, srLength + srSpace))

            print "maxSrLength = %g"%(maxSrLength)
            print "maxSpacing = %g"%(maxSpacing)

            # need to create new objects as python uses references
            seg0 = Point(p0.getX(), p0.getY())
            seg1 = Point(p1.getX(), p1.getY())

            # assumptions 1) and 2) above are pertinent here.
            if seqNext:
                print "a segment follows this one... lengthen for endcap"
                if dir == 'xdir':
                    seg1.setX(seg1.getX() + width/2.0)
                else:
                    seg1.setY(seg1.getY() + width/2.0)

            if seqPrev:
                print "this segment follows prior... shorten for prior seg"
                if dir == 'xdir':
                    seg0.setX(seg0.getX() + width/2.0)
                else:
                    seg0.setY(seg0.setY() + width/2.0)

            numrects = 0
            # equally distribute
            adjSpacing = maxSpacing
            if dir == 'xdir':
                numrects = int((seg1.getX() - seg0.getX() + srBeginOffset + srEndOffset)/maxSpacing)
                # ignore endOffset for now...
                adjSpacing = float((seg1.getX() - seg0.getX() + srBeginOffset)/(numrects - 1))
            else:
                numrects = int((seg1.getY() - seg0.getY() + srBeginOffset + srEndOffset)/maxSpacing)
                # ignore endOffset for now...
                adjSpacing = float((seg1.getY() - seg0.getY() + srBeginOffset)/(numrects - 1))
            if numrects == 0:
                continue

            print "number of subrects: %d"%(numrects)

            #
            # go through and create all the objects
            #
            # cp is the center point for placing the rects.
            #
            if dir == 'xdir':
                cp = seg0 + Point(-srBeginOffset + maxSrLength/2.0, 0.0)
            else:
                cp = seg0 + Point(0.0, -srBeginOffset + maxSrLength/2.0)

            print "starting points:"
            print "seg0: ", seg0
            print "seg1: ", seg1
            print "cp: ", cp

            for i in range(0, numrects - 1):
                srlayer = None
                srLength = 0
                srWidth = 0
                srSpace = 0
                srBeginOffset = 0
                srEndOffset = 0
                for sr in subRect:
                    for par in sr:
                        # assume the parameters are passed in a tuple
                        param,val = par
                        if param == "layer":
                            srLayer = val
                        elif param == "length":
                            srLength = val
                        elif param == "width":
                            srWidth = val
                        elif param == "space":
                            srSpace = val
                        elif param == "beginOffset":
                            srBeginOffset = val
                        elif param == "endOffset":
                            srEndOffset = val
                        
                    # initialize dependent parameters
                    if srLength == 0:
                        srLength = srWidth
                    if srWidth == 0:
                        srWidth = srLength
                    if srEndOffset == 0:
                        srEndOffset = srBeginOffset
                    if srBeginOffset == 0:
                        srBeginOffset = srEndOffset
                        
                    cr = Rect(Layer(srLayer), Box(0, 0, srWidth, srLength))
                    self.moveTo(cr, cp)
                    g.add(cr)

                print "placed subrect(s) at: ", cp

                if dir == 'xdir':
                    cp = cp + Point(adjSpacing, 0.0)
                else:
                    cp = cp + Point(0.0, adjSpacing)
                        
            p0 = p1 # reset this segment end to beginning of segment
            
        return g

"""

moduleDefRodCreateRect = """
    def rodCreateRect(self, layer=None, length=None, width=None):
        if layer == None or length == None or width == None:
            print "error in call to rodAlign"
            return
        r = Rect(Layer(layer), Box(Point(-width/2, -length/2), Point(length/2, width/2)))
        return r
        
"""

moduleDefRodAlign = """
    def rodAlign(self, refObj, refHandle, alignObj, alignHandle,  xSep=0, ySep=0):
        if refObj == None:
            print "error in call to rodAlign.  refObj == None"
            return
        if refObj == None or refHandle == None or alignObj == None or alignHandle == None:
            print "error in call to rodAlign.  something not assigned."
            return
        print "rodAlign: "
        print "ref handle", refHandle
        print "align handle", alignHandle
        p1 = refObj.getBBox().getHandlePoint(refHandle)
        print "old position: ", p1
        p1 = p1 + Point(xSep, ySep)
        print "new position: ", p1
        self.alignLocationToPoint(alignObj, alignHandle, p1)

"""
